@@ -1,4 +1,4 @@
-# $Id: Constants.pm,v 1.3 2001/02/22 01:15:45 btrott Exp $
+# $Id: Constants.pm,v 1.4 2001/03/03 05:31:43 btrott Exp $
 
 package Net::SSH::Perl::Constants;
 use strict;
@@ -29,6 +29,7 @@ use constant SSH_MSG_IGNORE => 32;
 use constant SSH_CMSG_EXIT_CONFIRMATION => 33;
 use constant SSH_CMSG_AUTH_RHOSTS_RSA => 35;
 use constant SSH_MSG_DEBUG => 36;
+use constant SSH_CMSG_REQUEST_COMPRESSION => 37;
 
 use constant HOST_OK => 1;
 use constant HOST_NEW => 2;
@@ -120,7 +121,8 @@ SSH_CMSG_EXEC_SHELL, SSH_CMSG_EXEC_CMD, SSH_SMSG_SUCCESS,
 SSH_SMSG_FAILURE, SSH_CMSG_STDIN_DATA, SSH_SMSG_STDOUT_DATA,
 SSH_SMSG_STDERR_DATA, SSH_CMSG_EOF, SSH_SMSG_EXITSTATUS,
 SSH_MSG_IGNORE, SSH_CMSG_EXIT_CONFIRMATION,
-SSH_CMSG_AUTH_RHOSTS_RSA, SSH_MSG_DEBUG.
+SSH_CMSG_AUTH_RHOSTS_RSA, SSH_MSG_DEBUG,
+SSH_CMSG_REQUEST_COMPRESSION.
 
 =item * hosts
 
